import remark from 'remark';
import remark_directive from 'remark-directive';
import markdown from 'remark-parse';
import frontmatter from 'remark-frontmatter';
import unified from 'unified';
import vfile, { VFile } from 'vfile';
import dedent from 'dedent';
import markdownFrontmatterLinterPlugin from '../index';
import mockFs from 'mock-fs';
import fs from 'fs';

const processMarkdown = async (md, opts): Promise<VFile> => {
    return await remark()
        .use(markdown)
        .use(frontmatter, ['yaml'])
        .use(markdownFrontmatterLinterPlugin as unified.Attacher, opts)
        .use(remark_directive)
        .processSync(md);
};

function loadFile(file: string): string {
    return fs.readFileSync(file, 'utf8');
}

describe('sfdocs-lint:remark-lint-markdown-frontmatter-playground', () => {
    beforeEach(() => {
        mockFs({
            '/test/content/guides/button': {
                'button-basic.md': dedent(`
                    ---
                    examples:
                    - name: basic
                      label: Basic Button
                      description: Button variants display the buttons with different colors to convey different meanings. The default variant is neutral.
                    ---
                    This is valid Markdown with correct front-matter.
                `),
                examples: {
                    basic: {
                        'basic.html': {}
                    },
                    disabled: {}
                },
            },
        });
    });

    afterEach(() => {
        mockFs.restore();
    });

    test('should not report an error if there is no front matter', async() => {
        const markdownContent = dedent(`# Title`);
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(0);
    });

    test('should not return errors for valid front-matter', async () => {
        const file = loadFile('/test/content/guides/button/button-basic.md');
        const result = await processMarkdown(file, '/test/content/guides/button');
        expect(result.messages).toHaveLength(0);
    });

    test('should report an error if there is no label in front-matter', async() => {
        const markdownContent = dedent(`
            ---
            examples:
                - name: basic
                  description: Button variants display the buttons with different colors to convey different meanings. The default variant is neutral.
            ---
        `)
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(1);
        expect(result.messages[0].toString()).toBe('/test/content/guides/button/button-basic.md:1:1-5:4: Playground example front matter is invalid.');
    });

    test('should report an error if there is no name in front-matter', async() => {
        const markdownContent = dedent(`
            ---
            examples:
                - label: Basic Button
                  description: Button variants display the buttons with different colors to convey different meanings. The default variant is neutral.
            ---
        `)
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(1);
        expect(result.messages[0].toString()).toBe('/test/content/guides/button/button-basic.md:1:1-5:4: Playground example front matter is invalid.');
    });

    test('should report an error if there is no description in front-matter', async() => {
        const markdownContent = dedent(`
            ---
            examples:
                - name: basic
                  label: Basic Button
            ---
        `)
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(1);
        expect(result.messages[0].toString()).toBe('/test/content/guides/button/button-basic.md:1:1-5:4: Playground example front matter is invalid.');
    });

    test('should report an error if there is no examples folder in file system', async() => {
        const markdownContent = dedent(`
            ---
            examples:
                - name: someButton
                  label: Basic Button
                  description: Button variants display the buttons with different colors to convey different meanings. The default variant is neutral.
            ---
        `)
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(1);
        expect(result.messages[0].toString()).toBe('/test/content/guides/button/button-basic.md:1:1-6:4: Folder \'someButton\' is missing but referenced in the playground example front matter.');
    });

    test('should report an error if there is no examples files in file system', async() => {
        const markdownContent = dedent(`
            ---
            examples:
                - name: disabled
                  label: Disabled Button
                  description: Button variants display the buttons with different colors to convey different meanings. The default variant is neutral.
            ---
        `)
        const file = vfile({ path: '/test/content/guides/button/button-basic.md', contents: markdownContent });
        const result = await processMarkdown(file, '/test/content/guides/button/button-basic.md');

        expect(result.messages).toHaveLength(1);
        expect(result.messages[0].toString()).toBe('/test/content/guides/button/button-basic.md:1:1-6:4: Folder \'/test/content/guides/button/examples/disabled\' is empty. Add the respective example files.');
    });
});