import visit from 'unist-util-visit';
import { VFile } from 'vfile';
import rule from 'unified-lint-rule';
import grayMatter from 'gray-matter';
import Ajv from 'ajv';
import path from 'path';
import fs from 'fs';

// JSON schema for front-matter validation
const playgroundExampleSchema = {
    type: 'object',
    properties: {
        examples: {
            type: 'array',
            items: {
                type: 'object',
                properties: {
                    name: { type: 'string' },
                    label: { type: 'string'},
                    description: { type: 'string'},
                },
                required: ['name', 'label', 'description'],
                additionalProperties: false,
            },
        },
    },
    required: ['examples'],
};

const ajv = new Ajv();
const validateSchema = ajv.compile(playgroundExampleSchema);

function checkForExampleFolder(node, file, examples, cwd) {
    const fileDirectory = path.resolve(file.path ? path.dirname(file.path) : cwd);
    const examplesDirectory = path.join(fileDirectory, 'examples');

    // Check if specific files exist based on front matter content
    examples.forEach((example: any) => {
        const folderName = example.name;
        const folderPath = path.join(examplesDirectory, folderName);

        try {
            if (!fs.existsSync(folderPath)) {
                file.message(
                    `Folder '${folderName}' is missing but referenced in the playground example front matter.`,
                    node
                );
                return;
            }
            const filesInExampleDir = fs.readdirSync(folderPath);
            if(filesInExampleDir.length === 0) {
                file.message(
                    `Folder '${folderPath}' is empty. Add the respective example files.`,
                    node
                );
            }
        } catch (err) {
            file.message(`Error checking example folder: '${folderPath}'`);
        }
    });
}

function validateFrontMatterInMarkdown(tree: any, file: VFile, cwd: string): void {
    visit(tree, ['yaml'], (node: any) => {
        const fileContent = file.toString();

        //Reg-ex to check if the example front matter exists or no.
        const frontMatterMatch = fileContent.match(/---\n([\s\S]+?)\n---/);
        if (!frontMatterMatch) {
            return;
        }

        const { data } = grayMatter(fileContent);
        if(!validateSchema(data)) {
            file.message(`Playground example front matter is invalid.`, node);
            return;
        }

        checkForExampleFolder(node, file, data?.examples, cwd);
    });
}

export = rule('sfdocs-lint:remark-lint-markdown-frontmatter-playground', validateFrontMatterInMarkdown);